/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import {
  VorgangMoreMenuE2EComponent,
  VorgangMoreMenuExportierenItemE2EComponent,
} from 'apps/alfa-e2e/src/components/vorgang/vorgang-more-menu.e2e.components';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import {
  countDownloadFiles,
  deleteDownloadFolder,
  dropCollections,
} from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgaenge } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang exportieren', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangMoreMenu: VorgangMoreMenuE2EComponent = vorgangPage.getMoreMenu();

  const vorgangExportieren: VorgangE2E = { ...createVorgang(), name: 'DoExportieren' };

  before(() => {
    initVorgaenge([vorgangExportieren]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('exportieren', () => {
    const menuItem: VorgangMoreMenuExportierenItemE2EComponent =
      vorgangMoreMenu.getExportierenItem();

    it('Open Vorgang-Detail-Page', () => {
      vorgangList.getListItem(vorgangExportieren.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('open "more menu"', () => {
      vorgangMoreMenu.getButton().click();

      exist(vorgangMoreMenu.getRoot());
    });

    it('should show item', () => {
      exist(menuItem.getRoot());
    });

    it('should show "Herunterladen" button', () => {
      exist(menuItem.getButton());
    });

    it('should have 1 file in download folder after download', () => {
      deleteDownloadFolder();
      menuItem.getButton().click();

      waitForSpinnerToDisappear();
      countDownloadFiles().then((count) => {
        expect(count).to.eq(1);
      });
    });

    it('should close menu after download', () => {
      notExist(menuItem.getRoot());
    });
  });
});
