import { VorgangListE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-list.e2e.component';
import { WiedervorlageInVorgangE2EComponent } from 'apps/alfa-e2e/src/components/wiedervorlage/wiedervorlage-in-vorgang.e2e.component';
import { WiedervorlagenInVorgangE2EComponent } from 'apps/alfa-e2e/src/components/wiedervorlage/wiedervorlagen-in-vorgang.e2e.component';
import { VorgangE2E } from 'apps/alfa-e2e/src/model/vorgang';
import { VorgangAttachedItemE2E } from 'apps/alfa-e2e/src/model/vorgang-attached-item';
import { WiedervorlageE2E } from 'apps/alfa-e2e/src/model/wiedervorlage';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { VorgangPage } from 'apps/alfa-e2e/src/page-objects/vorgang.po';
import { WiedervorlagePage } from 'apps/alfa-e2e/src/page-objects/wiedervorlage.po';
import { dropCollections } from 'apps/alfa-e2e/src/support/cypress-helper';
import { exist, haveText } from 'apps/alfa-e2e/src/support/cypress.util';
import { loginAsSabine } from 'apps/alfa-e2e/src/support/user-util';
import { initVorgangAttachedItem } from 'apps/alfa-e2e/src/support/vorgang-attached-item-util';
import {
  buildVorgang,
  createVorgang,
  initVorgaenge,
  objectIds,
} from 'apps/alfa-e2e/src/support/vorgang-util';
import {
  createWiedervorlageAttachedItem,
  createWiedervorlageItem,
} from 'apps/alfa-e2e/src/support/wiedervorlage-util';

describe('Vorgang wiedervorlage loading', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const wiedervorlagContainer: WiedervorlagenInVorgangE2EComponent =
    vorgangPage.getWiedervorlagenContainer();

  const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();

  const vorgangWithWiedervorlagen: VorgangE2E = {
    ...createVorgang(),
    name: 'VorgangWithWiedervorlagen',
  };
  const wiedervorlage: WiedervorlageE2E = createWiedervorlageItem('EineWiedervorlage');
  const wiedervorlageAttachedItem: VorgangAttachedItemE2E = {
    ...createWiedervorlageAttachedItem(objectIds[0], vorgangWithWiedervorlagen._id.$oid),
    item: wiedervorlage,
  };

  const vorgangWithOtherWiedervorlagen: VorgangE2E = buildVorgang(
    objectIds[0],
    'VorgangWithOtherWiedervorlagen',
  );
  const otherWiedervorlage: WiedervorlageE2E = createWiedervorlageItem('EineAndereWiedervorlage');
  const otherWiedervorlageAttachedItem: VorgangAttachedItemE2E = {
    ...createWiedervorlageAttachedItem(objectIds[1], vorgangWithOtherWiedervorlagen._id.$oid),
    item: otherWiedervorlage,
  };

  before(() => {
    initVorgaenge([vorgangWithWiedervorlagen, vorgangWithOtherWiedervorlagen]);
    initVorgangAttachedItem([wiedervorlageAttachedItem, otherWiedervorlageAttachedItem]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('Vorgang with wiedervorlagen', () => {
    it('should show vorgang detail by click on vorgang in list', () => {
      vorgangList.getListItem(vorgangWithWiedervorlagen.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show wiedervorlage on vorgang page', () => {
      const singleWiedervorlage = wiedervorlagContainer.getWiedervorlage(wiedervorlage.betreff);

      exist(singleWiedervorlage.getRoot());

      haveText(singleWiedervorlage.getBetreff(), wiedervorlage.betreff);
    });

    it('should open Wiedervorlage-Page by click on wiedervorlage ', () => {
      const wiedervorlageInVorgang: WiedervorlageInVorgangE2EComponent =
        wiedervorlagContainer.getWiedervorlage(wiedervorlage.betreff);

      wiedervorlageInVorgang.getLink().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlagePage.getSubnavigation().getRoot());
    });

    it('should show wiedervorlage on vorgang page on navigate back', () => {
      wiedervorlagePage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());

      const singleWiedervorlage = wiedervorlagContainer.getWiedervorlage(wiedervorlage.betreff);
      exist(singleWiedervorlage.getRoot());
    });

    it('should show vorganglist on navigate back', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });
  });

  describe.skip('Vorgang without wiedervorlagen', () => {
    it('should show vorgang detail by click on vorgang in list', () => {
      vorgangList.getListItem(vorgangWithOtherWiedervorlagen.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show other wiedervorlage on vorgang page', () => {
      const singleWiedervorlage = wiedervorlagContainer.getWiedervorlage(
        otherWiedervorlage.betreff,
      );

      exist(singleWiedervorlage.getRoot());

      haveText(singleWiedervorlage.getBetreff(), otherWiedervorlage.betreff);
    });

    it('should open Wiedervorlage-Page by click on wiedervorlage ', () => {
      const wiedervorlageInVorgang: WiedervorlageInVorgangE2EComponent =
        wiedervorlagContainer.getWiedervorlage(otherWiedervorlage.betreff);

      wiedervorlageInVorgang.getLink().click();
      waitForSpinnerToDisappear();

      exist(wiedervorlagePage.getSubnavigation().getRoot());
    });

    it('should keep empty wiedervorlagen on vorgang page on navigate back', () => {
      wiedervorlagePage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());

      const singleWiedervorlage = wiedervorlagContainer.getWiedervorlage(
        otherWiedervorlage.betreff,
      );
      exist(singleWiedervorlage.getRoot());
    });

    it('should show vorganglist on navigate back', () => {
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });
  });
});
