/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UserE2E } from '../model/user';
import { UsermanagerUserE2E } from '../model/usermanager';
import { getCypressEnv, initUsermanagerData, login } from './cypress-helper';

const sabineFixture: UserE2E = require('../fixtures/user-main/user_sabine.json');
const dorotheaFixture: UserE2E = require('../fixtures/user-main/user_dorothea.json');

const userManagerSabineFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_sabine.json');
const userManagerPeterFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_peter.json');
const userManagerEmilFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_emil.json');
const userManagerDorotheaFixture: UsermanagerUserE2E = require('../fixtures/usermanager/usermanager_user_dorothea.json');

export function initUsermanagerUsers(users: UsermanagerUserE2E[]) {
  initUsermanagerData(users);
}

export function getUserSabine(): UserE2E {
  return sabineFixture;
}

export function getUserDorothea(): UserE2E {
  return dorotheaFixture;
}

export function getUserManagerUserSabine(): UsermanagerUserE2E {
  return withIdsFromCypressEnv(userManagerSabineFixture);
}

export function getUserManagerUserPeter(): UsermanagerUserE2E {
  return withIdsFromCypressEnv(userManagerPeterFixture);
}

export function getUserManagerUserEmil(): UsermanagerUserE2E {
  return withIdsFromCypressEnv(userManagerEmilFixture);
}

export function getUserManagerUserDorothea(): UsermanagerUserE2E {
  return withIdsFromCypressEnv(userManagerDorotheaFixture);
}

export function getUserSabineInternalId(): string {
  return getUserManagerUserSabine()._id.$oid;
}

function withIdsFromCypressEnv(user: UsermanagerUserE2E): UsermanagerUserE2E {
  user._id = { $oid: getUserInternalIdFromCypressEnv(user.username) };
  user.keycloakUserId = getKeycloakUserIdFromCypressEnv(user.username);
  return user;
}

function getUserInternalIdFromCypressEnv(username: string): string {
  return getCypressEnv(`${username}InternalId`);
}

function getKeycloakUserIdFromCypressEnv(username: string): string {
  return getCypressEnv(`${username}Uuid`);
}

enum DatabaseUser {
  EMIL = 'user-ea/user_emil.json',
  KFINDER = 'user-main/user_kfinder.json',
  KORDNER = 'user-main/user_kordner.json',
  LUDWIG = 'user-main/user_ludwig.json',
  PETER = 'user-main/user_peter.json',
  RICHARD = 'user-main/user_richard.json',
  SABINE = 'user-main/user_sabine.json',
  ZONK = 'user-main/user_zonk.json',
}

export function loginAsEmil(): void {
  login(DatabaseUser.EMIL);
}

export function loginAsKfinder(): void {
  login(DatabaseUser.KFINDER);
}

export function loginAsKordner(): void {
  login(DatabaseUser.KORDNER);
}

export function loginAsPeter(): void {
  login(DatabaseUser.PETER);
}

export function loginAsRichard(): void {
  login(DatabaseUser.RICHARD);
}

export function loginAsSabine(): void {
  login(DatabaseUser.SABINE);
}

export function loginAsZonk(): void {
  login(DatabaseUser.ZONK);
}

export function loginAsLudwig(): void {
  login(DatabaseUser.LUDWIG);
}

export enum UserRoleE2E {
  EINHEITLICHER_ANSPRECHPARTNER = 'EINHEITLICHER_ANSPRECHPARTNER',
  VERWALTUNG_LOESCHEN = 'VERWALTUNG_LOESCHEN',
  VERWALTUNG_POSTSTELLE = 'VERWALTUNG_POSTSTELLE',
  VERWALTUNG_USER = 'VERWALTUNG_USER',
}

//TODO Cleanup
export function loginByUi(user: UserE2E): void {
  cy.visit('')
    .get('#kc-login')
    .should('exist')
    .get('#username')
    .type(user.name)
    .get('#password')
    .type(user.password)
    .get('#kc-login')
    .click();
}
