import { ComponentFixture, TestBed } from '@angular/core/testing';
import { Mock, dispatchEventFromFixture, mock } from '@alfa-client/test-utils';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { BescheidService } from 'libs/bescheid-shared/src/lib/bescheid.service';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { CreateBescheidButtonContainerComponent } from './create-bescheid-button-container.component';
import { CreateBescheidButtonComponent } from './create-bescheid-button/create-bescheid-button.component';

describe('CreateBescheidButtonContainerComponent', () => {
  let component: CreateBescheidButtonContainerComponent;
  let fixture: ComponentFixture<CreateBescheidButtonContainerComponent>;

  const createBescheidComponent: string = getDataTestIdOf('create-bescheid-button-component');
  const service: Mock<BescheidService> = mock(BescheidService);
  const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        CreateBescheidButtonContainerComponent,
        MockComponent(CreateBescheidButtonComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: service,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(CreateBescheidButtonContainerComponent);
    component = fixture.componentInstance;
    component.vorgangWithEingang = vorgangWithEingang;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should get bescheid command', () => {
      component.ngOnInit();

      expect(service.getBescheidCommand).toHaveBeenCalled();
    });
  });

  describe('create', () => {
    it('should call service', () => {
      dispatchEventFromFixture(fixture, createBescheidComponent, 'createBescheid');

      expect(service.createBescheid).toHaveBeenCalledWith(vorgangWithEingang);
    });
  });
});
