import { CommandResource, CreateCommand } from '@alfa-client/command-shared';
import {
  ApiError,
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Action } from '@ngrx/store';
import { Resource, ResourceUri } from '@ngxp/rest';
import { createCommandResource, createCreateCommand } from 'libs/command-shared/test/command';
import { createApiError } from 'libs/tech-shared/test/error';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { CommandState, initialState, reducer } from './command.reducer';

import faker from '@faker-js/faker';

import * as CommandActions from '@alfa-client/command-shared';

describe('Command Reducer', () => {
  describe('unknown action', () => {
    it('should return current state', () => {
      const action: Action = {} as Action;

      const result: any = reducer(initialState, action);

      expect(result).toBe(initialState);
    });
  });

  describe('createCommand', () => {
    const resource: Resource = createDummyResource();
    const linkRel: ResourceUri = faker.internet.url();
    const command: CreateCommand = createCreateCommand();

    it('should create empty loading map entry', () => {
      const action = CommandActions.createCommand({ resource, linkRel, command });

      const state: CommandState = reducer(initialState, action);

      expect(state.commandByOrderMap[command.order]).toEqual(createEmptyStateResource(true));
    });
  });

  describe('createCommandSuccess', () => {
    const command: CommandResource = createCommandResource();

    it('should create stateResource entry by loaded resource', () => {
      const action = CommandActions.createCommandSuccess({ command });

      const state: CommandState = reducer(initialState, action);

      expect(state.commandByOrderMap[command.order]).toEqual(createStateResource(command));
    });
  });

  describe('createCommandFailure', () => {
    const command: CommandResource = createCommandResource();
    const error: ApiError = createApiError();

    it('should create errorStateResource entry by occured error', () => {
      const action = CommandActions.createCommandFailure({ command, error });

      const state: CommandState = reducer(initialState, action);

      expect(state.commandByOrderMap[command.order]).toEqual(createErrorStateResource(error));
    });
  });
});
