/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { getEmbeddedResource } from '@ngxp/rest';
import { DummyListLinkRel } from 'libs/tech-shared/test/dummy';
import { createDummyListResource } from 'libs/tech-shared/test/resource';
import { ToEmbeddedResourcesPipe } from './to-embedded-resource.pipe';

describe('ToEmbeddedResourcesPipe', () => {
  const pipe = new ToEmbeddedResourcesPipe();

  it('create an instance', () => {
    expect(pipe).toBeTruthy();
  });

  describe('transform listResource to resources by link', () => {
    const listResource = createDummyListResource([DummyListLinkRel.LIST]);

    it('should return embedded resources', () => {
      const result = pipe.transform(listResource, DummyListLinkRel.LIST);

      expect(result).toBe(getEmbeddedResource(listResource, DummyListLinkRel.LIST));
    });

    it('should return null by empty listResource', () => {
      const result = pipe.transform(null, DummyListLinkRel.LIST);

      expect(result).toBe(null);
    });

    it('should return null by empty linkel', () => {
      const result = pipe.transform(listResource, null);

      expect(result).toBe(null);
    });
  });
});
