import { ComponentFixture, TestBed } from '@angular/core/testing';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { dispatchEventFromFixture, mock } from '@alfa-client/test-utils';
import { OzgcloudButtonWithSpinnerComponent } from '@alfa-client/ui';
import { VorgangService } from '@alfa-client/vorgang-shared';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { VorgangExportContainerComponent } from './vorgang-export-container.component';

describe('VorgangExportContainerComponent', () => {
  let component: VorgangExportContainerComponent;
  let fixture: ComponentFixture<VorgangExportContainerComponent>;

  const vorgangExportStateResource: StateResource<boolean> = createStateResource(false);
  const vorgangService = mock(VorgangService);

  const buttonWithSpinner: string = '[dataTestId="export-vorgang"]';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangExportContainerComponent,
        MockComponent(OzgcloudButtonWithSpinnerComponent),
      ],
      providers: [
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangExportContainerComponent);
    component = fixture.componentInstance;
    vorgangService.getVorgangExport.mockReturnValue(of(vorgangExportStateResource));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should get export', () => {
      component.ngOnInit();

      expect(vorgangService.getVorgangExport).toHaveBeenCalled();
    });
  });

  describe('isExportDone', () => {
    it('should closeMenu if export is done', () => {
      jest.spyOn(component.closeMenu, 'emit');

      component.isExportDone(createStateResource(true));

      expect(component.closeMenu.emit).toHaveBeenCalled();
    });
  });

  describe('export', () => {
    it('should call vorgangService.export', () => {
      dispatchEventFromFixture(fixture, buttonWithSpinner, 'clickEmitter');

      expect(vorgangService.export).toHaveBeenCalled();
    });
  });
});
