/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandOrder } from '@alfa-client/command-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { createCreateCommand } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { VorgangFacade } from './+state/vorgang.facade';
import { VorgangCommandService } from './vorgang-command.service';
import { VorgangWithEingangResource } from './vorgang.model';

describe('VorgangCommandService', () => {
  let service: VorgangCommandService;
  let facade: Mock<VorgangFacade>;

  const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

  beforeEach(() => {
    facade = mock(VorgangFacade);

    service = new VorgangCommandService(useFromMock(facade));
  });

  describe('getAnnehmenCommand', () => {
    it('should call facade', () => {
      service.getAnnehmenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_ANNEHMEN);
    });
  });

  describe('annehmen', () => {
    it('should call facade', () => {
      service.annehmen(vorgangWithEingang);

      expect(facade.annehmen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_ANNEHMEN),
      );
    });
  });

  describe('getBearbeitenCommand', () => {
    it('should call facade', () => {
      service.getBearbeitenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_BEARBEITEN);
    });
  });

  describe('bearbeiten', () => {
    it('should call facade', () => {
      service.bearbeiten(vorgangWithEingang);

      expect(facade.bearbeiten).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_BEARBEITEN),
      );
    });
  });

  describe('getBeschiedenCommand', () => {
    it('should call facade', () => {
      service.getBeschiedenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_BESCHEIDEN);
    });
  });

  describe('bescheiden', () => {
    it('should call facade', () => {
      service.bescheiden(vorgangWithEingang);

      expect(facade.bescheiden).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_BESCHEIDEN),
      );
    });
  });

  describe('getVerwerfenCommand', () => {
    it('should call facade', () => {
      service.getVerwerfenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_VERWERFEN);
    });
  });

  describe('verwerfen', () => {
    it('should call facade', () => {
      service.verwerfen(vorgangWithEingang);

      expect(facade.verwerfen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_VERWERFEN),
      );
    });
  });

  describe('getWiedereroeffnenCommand', () => {
    it('should call facade', () => {
      service.getWiedereroeffnenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_WIEDEREROEFFNEN);
    });
  });

  describe('wiedereroeffnen', () => {
    it('should call facade', () => {
      service.wiedereroeffnen(vorgangWithEingang);

      expect(facade.wiedereroeffnen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_WIEDEREROEFFNEN),
      );
    });
  });

  describe('getZurueckholenCommand', () => {
    it('should call facade', () => {
      service.getZurueckholenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_ZURUECKHOLEN);
    });
  });

  describe('zurueckholen', () => {
    it('should call facade', () => {
      service.zurueckholen(vorgangWithEingang);

      expect(facade.zurueckholen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_ZURUECKHOLEN),
      );
    });
  });

  describe('getZurueckstellenCommand', () => {
    it('should call facade', () => {
      service.getZurueckstellenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_ZURUECKSTELLEN);
    });
  });

  describe('zurueckstellen', () => {
    it('should call facade', () => {
      service.zurueckstellen(vorgangWithEingang);

      expect(facade.zurueckstellen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_ZURUECKSTELLEN),
      );
    });
  });

  describe('getAbschliessenCommand', () => {
    it('should call facade', () => {
      service.getAbschliessenCommand();

      expect(facade.getStatusCommand).toHaveBeenCalledWith(CommandOrder.VORGANG_ABSCHLIESSEN);
    });
  });

  describe('abschliessen', () => {
    it('should call facade', () => {
      service.abschliessen(vorgangWithEingang);

      expect(facade.abschliessen).toHaveBeenCalledWith(
        vorgangWithEingang,
        createCreateCommand(CommandOrder.VORGANG_ABSCHLIESSEN),
      );
    });
  });

  describe('getRevokeCommand', () => {
    it('should call facade', () => {
      service.getRevokeCommand();

      expect(facade.getRevokeCommand).toHaveBeenCalled();
    });
  });
});
