/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MatIconTestingModule } from '@angular/material/icon/testing';
import {
  FormatDateWithTimePipe,
  FormatDateWithoutYearWithTimePipe,
  formatDateWithoutYearWithTime,
} from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { VorgangResource } from '@alfa-client/vorgang-shared';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MatTooltipDirective } from 'libs/ui/src/lib/ui/mattooltip/mattooltip.directive';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { VorgangCreatedAtComponent } from './vorgang-created-at.component';

registerLocaleData(localeDe);

describe('VorgangCreatedAtComponent', () => {
  let component: VorgangCreatedAtComponent;
  let fixture: ComponentFixture<VorgangCreatedAtComponent>;

  const vorgang: VorgangResource = createVorgangResource();
  const createdAt: string = getDataTestIdOf('created-at');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        MatIcon,
        MatTooltipDirective,
        FormatDateWithoutYearWithTimePipe,
        FormatDateWithTimePipe,
        VorgangCreatedAtComponent,
      ],
      imports: [MatIconTestingModule],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangCreatedAtComponent);
    component = fixture.componentInstance;
    component.vorgang = vorgang;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('created-at', () => {
    it('should show vorgang.createdAt', () => {
      const text: HTMLElement = getElementFromFixture(fixture, createdAt);
      const vorgangCreatedAtFormatted: string = formatDateWithoutYearWithTime(vorgang.createdAt);

      expect(text.textContent).toBe(vorgangCreatedAtFormatted);
    });
  });
});
