/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import java.util.HashSet;
import java.util.Set;
import java.util.stream.Stream;

import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.loeschanforderung.DeleteLoeschAnforderung;

@Service
class HistorieService {

	@Autowired
	private CommandService commandService;
	@Autowired
	private HistorieCommandHandler historieCommandHandler;

	public Stream<Command> findFinishedCommands(String vorgangId) {
		var reduceBy = new HashSet<String>();
		var commands = commandService.findFinishedCommands(vorgangId)
				.filter(historieCommandHandler::isHistorieCommand)
				.map(command -> considerReduceByIds(reduceBy, command))
				.map(historieCommandHandler::translateOrder).toList();

		return commands.stream().filter(command -> !reduceBy.contains(command.getId()));
	}

	private Command considerReduceByIds(Set<String> reduceBy, Command command) {
		if (isLoeschAnforderungZuruecknehmenCommand(command)) {
			reduceBy.addAll(getCommandBodyIds(command));
		}
		return command;
	}

	private boolean isLoeschAnforderungZuruecknehmenCommand(Command command) {
		return command.getOrder() == CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN;
	}

	private Set<String> getCommandBodyIds(Command command) {
		return Set.of(getDeleteAttachedItemCommandId(command), getChangeStatusCommandId(command));
	}

	private String getDeleteAttachedItemCommandId(Command command) {
		return MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.DELETE_ATTACHED_ITEM_COMMAND_ID_FIELD);
	}

	private String getChangeStatusCommandId(Command command) {
		return MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.CHANGE_STATUS_COMMAND_ID_FIELD);
	}
}