package de.ozgcloud.alfa.historie;

import java.util.Map;
import java.util.Optional;
import java.util.Set;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class StatusChangeHistoryBuilder extends ChangeHistoryBuilder<StatusChangeHistoryBuilder> {

	static final String GELOESCHT_NAME = "Gelöscht";
	static final Map<VorgangStatus, String> VORGANG_STATUS_TO_NAME = Map.of(
			VorgangStatus.NEU, "Neu",
			VorgangStatus.ANGENOMMEN, "Angenommen",
			VorgangStatus.VERWORFEN, "Verworfen",
			VorgangStatus.IN_BEARBEITUNG, "In Bearbeitung",
			VorgangStatus.BESCHIEDEN, "Beschieden",
			VorgangStatus.ABGESCHLOSSEN, "Abgeschlossen",
			VorgangStatus.WEITERGELEITET, "Weitergeleitet",
			VorgangStatus.ZU_LOESCHEN, "Zu Löschen"
	);
	static final Set<CommandOrder> STATUS_CHANGE_COMMAND_ORDER = Set.of(
			CommandOrder.VORGANG_ANNEHMEN,
			CommandOrder.VORGANG_VERWERFEN,
			CommandOrder.VORGANG_ZURUECKHOLEN,
			CommandOrder.VORGANG_BEARBEITEN,
			CommandOrder.VORGANG_BESCHEIDEN,
			CommandOrder.VORGANG_ABSCHLIESSEN,
			CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN,
			CommandOrder.VORGANG_WIEDEREROEFFNEN,
			CommandOrder.VORGANG_LOESCHEN,
			CommandOrder.VORGANG_ZURUECKSTELLEN);

	public static StatusChangeHistoryBuilder builder() {
		return new StatusChangeHistoryBuilder();
	}

	@Override
	boolean isRelevant(Command command) {
		return STATUS_CHANGE_COMMAND_ORDER.contains(command.getOrder());
	}

	@Override
	CommandWithChangeValues toCommandWithChangeValues(CommandWithPrevious commandWithPrevious) {
		return new CommandWithChangeValues(commandWithPrevious.getCommand(),
				getStatusBeforeChange(commandWithPrevious),
				getStatusAfterChange(commandWithPrevious));
	}

	String getStatusBeforeChange(CommandWithPrevious commandWithPrevious) {
		return Optional.ofNullable(commandWithPrevious.getPrevious())
				.map(this::getStatus)
				.orElse(VORGANG_STATUS_TO_NAME.get(VorgangStatus.NEU));
	}

	String getStatusAfterChange(CommandWithPrevious commandWithPrevious) {
		return getStatus(commandWithPrevious.getCommand());
	}

	String getStatus(Command command) {
		return switch (command.getOrder()) {
			case VORGANG_ANNEHMEN, VORGANG_ZURUECKSTELLEN -> VORGANG_STATUS_TO_NAME.get(VorgangStatus.ANGENOMMEN);
			case VORGANG_VERWERFEN -> VORGANG_STATUS_TO_NAME.get(VorgangStatus.VERWORFEN);
			case VORGANG_ZURUECKHOLEN -> VORGANG_STATUS_TO_NAME.get(VorgangStatus.NEU);
			case VORGANG_BEARBEITEN, VORGANG_WIEDEREROEFFNEN -> VORGANG_STATUS_TO_NAME.get(VorgangStatus.IN_BEARBEITUNG);
			case VORGANG_BESCHEIDEN -> VORGANG_STATUS_TO_NAME.get(VorgangStatus.BESCHIEDEN);
			case VORGANG_ABSCHLIESSEN -> VORGANG_STATUS_TO_NAME.get(VorgangStatus.ABGESCHLOSSEN);
			case VORGANG_ZUM_LOESCHEN_MARKIEREN -> VORGANG_STATUS_TO_NAME.get(VorgangStatus.ZU_LOESCHEN);
			case VORGANG_LOESCHEN -> GELOESCHT_NAME;
			default -> throw new IllegalArgumentException("Unexpected or unknown command order " + command.getOrder());
		};
	}
}
