package de.ozgcloud.alfa.file;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.PrimaerdokumentTypeBuilder;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.FormatType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.MediumCodeType;
import de.xoev.xdomea.PrimaerdokumentType;
import de.xoev.xdomea.VersionType;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class DokumentTypeBuilder {

	static final String VERSION_NUMMER = "1";
	static final String ALLGEMEINE_METADATEN_MEDIUM_CODE = "001";

	private Long ordinalNumber;
	private String formEngineName;
	private OzgFile ozgFile;

	public static DokumentTypeBuilder builder() {
		return new DokumentTypeBuilder();
	}

	public DokumentTypeBuilder withOrdinalNumber(Long ordinalNumber) {
		this.ordinalNumber = ordinalNumber;
		return this;
	}

	public DokumentTypeBuilder withFormEngineName(String formEngineName) {
		this.formEngineName = formEngineName;
		return this;
	}

	public DokumentTypeBuilder withOzgFile(OzgFile ozgFile) {
		this.ozgFile = ozgFile;
		return this;
	}

	public DokumentType build() {
		var dokumentType = new DokumentType();
		dokumentType.setIdentifikation(createIdentifikation());
		dokumentType.setAllgemeineMetadaten(createAllgemeineMetadaten());
		dokumentType.getVersion().add(createVersionType());
		dokumentType.getVersion().get(0).getFormat().get(0).setPrimaerdokument(createPrimaerdokument());
		return dokumentType;
	}

	VersionType createVersionType() {
		var versionType = new VersionType();
		versionType.setNummer(VERSION_NUMMER);
		versionType.getFormat().add(createFormatType());
		return versionType;
	}

	FormatType createFormatType() {
		var formatType = new FormatType();
		formatType.setName(DateiformatCode.PDF.createDateiformatCodeType());
		formatType.setSonstigerName(StringUtils.EMPTY);
		formatType.setVersion(StringUtils.EMPTY);
		return formatType;
	}

	IdentifikationObjektType createIdentifikation() {
		var identifikation = new IdentifikationObjektType();
		identifikation.setID(UUIDConverter.fromObjectId(ozgFile.getId().toString()));
		identifikation.setNummerImUebergeordnetenContainer(ordinalNumber);
		return identifikation;
	}

	AllgemeineMetadatenType createAllgemeineMetadaten() {
		var allgemeineMetadaten = new AllgemeineMetadatenType();
		allgemeineMetadaten.setBetreff(ozgFile.getName());
		allgemeineMetadaten.setKennzeichen(StringUtils.EMPTY);
		allgemeineMetadaten.setBemerkung(StringUtils.EMPTY);
		allgemeineMetadaten.setMedium(createMediumCode());
		return allgemeineMetadaten;
	}

	private MediumCodeType createMediumCode() {
		var mediumCode = new MediumCodeType();
		mediumCode.setCode(ALLGEMEINE_METADATEN_MEDIUM_CODE);
		return mediumCode;
	}

	PrimaerdokumentType createPrimaerdokument() {
		return PrimaerdokumentTypeBuilder.builder()
				.withOzgFile(ozgFile)
				.withErsteller(formEngineName)
				.build();
	}
}
