package de.ozgcloud.alfa.file;

import java.io.OutputStream;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicLong;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileService;
import de.ozgcloud.alfa.vorgang.Eingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.xoev.xdomea.DokumentType;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
public class ExportFileService {

	private static final Predicate<OzgFile> IS_PDF_FILE = file -> MediaType.APPLICATION_PDF_VALUE.equals(file.getContentType());

	private final OzgFileService ozgFileService;
	private final BinaryFileService binaryFileService;

	public Stream<OzgFile> getRepresentations(VorgangWithEingang vorgang) {
		return Optional.ofNullable(vorgang.getEingang())
				.map(Eingang::getId).stream()
				.flatMap(ozgFileService::getRepresentationsByEingang)
				.filter(IS_PDF_FILE);
	}

	public Stream<OzgFile> getAttachments(VorgangWithEingang vorgang) {
		return Optional.ofNullable(vorgang.getEingang())
				.map(Eingang::getId).stream()
				.flatMap(ozgFileService::getAttachmentsByEingang)
				.filter(IS_PDF_FILE);
	}

	public Stream<DokumentType> createDokumentTypes(List<OzgFile> ozgFiles, String formEngineName) {
		var ordinalNumberGenerator = new AtomicLong(1);
		return ozgFiles.stream().map(ozgFile -> DokumentTypeBuilder.builder()
				.withOrdinalNumber(ordinalNumberGenerator.getAndIncrement())
				.withFormEngineName(formEngineName)
				.withOzgFile(ozgFile)
				.build());
	}

	public void writeOzgFile(FileId fileId, OutputStream outputStream) {
		binaryFileService.writeFileContent(fileId, outputStream);
	}

}
