package de.ozgcloud.alfa.historie;

import java.util.function.Function;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.xoev.xdomea.HistorienProtokollInformationType;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
public class ExportHistorieService {

	private final VorgangChangeHistoryService vorgangChangeHistoryService;

	public Stream<HistorienProtokollInformationType> createHistorienProtokollInformationTypes(VorgangWithEingang vorgang) {
		var history = vorgangChangeHistoryService.createVorgangChangeHistory(vorgang);
		return Stream.of(
						history.getStatusChangeHistory().stream().map(this::createHistorienProtokollInformationType),
						history.getAktenzeichenChangeHistory().stream().map(this::createHistorienProtokollInformationType),
						history.getAssignedUserChangeHistory().stream().map(this::createHistorienProtokollInformationType))
				.flatMap(Function.identity());
	}

	HistorienProtokollInformationType createHistorienProtokollInformationType(VorgangChange vorgangChange) {
		var historienProtokollInformationType = new HistorienProtokollInformationType();
		historienProtokollInformationType.setMetadatumAlterWert(createValueBeforeChange(vorgangChange));
		historienProtokollInformationType.setMetadatumNeuerWert(createValueAfterChange(vorgangChange));
		historienProtokollInformationType.setAkteur(createAkteur(vorgangChange));
		historienProtokollInformationType.setDatumUhrzeit(DateConverter.toXmlGregorianCalendar(vorgangChange.getFinishedAt()));
		historienProtokollInformationType.setAktion(vorgangChange.getOrder().name());
		return historienProtokollInformationType;
	}

	String createAkteur(VorgangChange vorgangChange) {
		return appendOrganisationseinheitenID(vorgangChange.getAuthorFullName(), vorgangChange.getOrganisationseinheitenID());
	}

	String createValueBeforeChange(VorgangChange vorgangChange) {
		if (vorgangChange.getOrder() == CommandOrder.ASSIGN_USER) {
			return appendOrganisationseinheitenID(vorgangChange.getValueBeforeChange(), vorgangChange.getOrganisationseinheitenID());
		}
		return vorgangChange.getValueBeforeChange();
	}

	String createValueAfterChange(VorgangChange vorgangChange) {
		if (vorgangChange.getOrder() == CommandOrder.ASSIGN_USER) {
			return appendOrganisationseinheitenID(vorgangChange.getValueAfterChange(), vorgangChange.getOrganisationseinheitenID());
		}
		return vorgangChange.getValueAfterChange();
	}

	String appendOrganisationseinheitenID(String text, String organisationseinheitenID) {
		 return StringUtils.isNotBlank(text) ? text + "; " + organisationseinheitenID : StringUtils.EMPTY;
	}
}
