package de.ozgcloud.alfa.kommentar;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;

class DateiformatCodeTest {

	@Nested
	class TestGetXdomeaCode {

		@Test
		void shouldReturnCode() {
			var code = DateiformatCode.getXdomeaCode("application/pdf", "pdf");

			assertThat(code).isEqualTo("018");
		}

		@Test
		void shouldReturnSonstiges() {
			var code = DateiformatCode.getXdomeaCode("abc", "abc");

			assertThat(code).isEqualTo(DateiformatCode.SONSTIGES_CODE);
		}

		@ParameterizedTest
		@NullAndEmptySource
		void shouldReturnCodeOnSingleFileExtensionMapping(String extension) {
			var code = DateiformatCode.getXdomeaCode("application/pdf", extension);

			assertThat(code).isEqualTo("018");
		}

		@Test
		void shouldReturnCodeOnMultipleFileExtensionMapping() {
			var code = DateiformatCode.getXdomeaCode("image/jpeg", "jpg");

			assertThat(code).isEqualTo("011");
		}

		@ParameterizedTest
		@NullAndEmptySource
		void shouldReturnSonstigesOnMultipleFileExtensionMapping(String extension) {
			var code = DateiformatCode.getXdomeaCode("image/jpeg", extension);

			assertThat(code).isEqualTo(DateiformatCode.SONSTIGES_CODE);
		}

		@ParameterizedTest
		@NullAndEmptySource
		void shouldReturnSonstigesOnEmptyMimeType(String mimeType) {
			var code = DateiformatCode.getXdomeaCode(mimeType, "pdf");

			assertThat(code).isEqualTo(DateiformatCode.SONSTIGES_CODE);
		}
	}

}