{{/* vim: set filetype=mustache: */}}

{{/* Truncate at 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec) */}}
{{/* Name */}}
{{- define "app.name" -}}
{{- default .Release.Name | toString | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/* Namespace */}}
{{- define "app.namespace" -}}
{{- default .Release.Namespace | toString | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/* Version */}}
{{- define "app.version" -}}
{{- default .Chart.Version | toString | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/* Chart: Name + Version */}}
{{- define "app.chart" -}}
{{ printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" }}
{{- end -}}

{{/* Managed-by -> On Helm, this value is always Helm */}}
{{- define "app.managedBy" -}}
{{- default .Release.Service | toString | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/* Default Labels: Helm recommended best-practice labels https://helm.sh/docs/chart_best_practices/labels/ */}}
{{- define "app.defaultLabels" }}
app.kubernetes.io/instance: alfa
app.kubernetes.io/managed-by: {{ include "app.managedBy" . }}
app.kubernetes.io/name: {{ include "app.name" . }}
app.kubernetes.io/part-of: ozgcloud
app.kubernetes.io/version: {{ include "app.version" . }}
app.kubernetes.io/namespace: {{ include "app.namespace" . }}
helm.sh/chart: {{ include "app.chart" . }}
{{- end -}}

{{- define "app.matchLabels" }}
app.kubernetes.io/name: {{ include "app.name" . }}
app.kubernetes.io/namespace: {{ include "app.namespace" . }}
{{- end -}}

{{- define "app.envSpringProfiles" }}
{{- if (.Values.env).overrideSpringProfiles -}}
{{ printf "%s" (.Values.env).overrideSpringProfiles }}
{{- else -}}
{{ printf "oc, %s" (include "app.ozgcloudEnvironment" . ) }}
{{- end -}}
{{- end -}}

{{- define "app.grpc_client_vorgang_manager_address" -}}
{{ printf "%s.%s:9090" ( coalesce .Values.vorgangManagerName "vorgang-manager" ) .Release.Namespace | replace "+" "_" | trunc 63 | trimSuffix "-" }}
{{- end -}}

{{- define "app.grpc_client_user-manager_address" -}}
{{ printf "%s.%s:9000" ( coalesce .Values.usermanagerName "user-manager" ) .Release.Namespace }}
{{- end -}}

{{- define "app.baseUrl" -}}
{{- required "baseUrl muss angegeben sein" .Values.baseUrl }}
{{- end -}}

{{- define "app.ozgcloud_user-manager_url" -}}
{{- if (.Values).usermanagerUrl -}}
{{ printf "%s" (.Values).usermanagerUrl }}
{{- else -}}
{{ printf "https://%s-%s.%s" (include "app.ozgcloudBezeichner" .) .Values.usermanagerName (include "app.baseUrl" .) }}
{{- end -}}
{{- end -}}

{{- define "app.imagePullSecret" }}
{{- with .Values.imageCredentials }}
{{- printf "{\"auths\":{\"%s\":{\"username\":\"%s\",\"password\":\"%s\",\"email\":\"%s\",\"auth\":\"%s\"}}}" .registry .username .password .email (printf "%s:%s" .username .password | b64enc) | b64enc }}
{{- end }}
{{- end }}

{{- define "app.ozgcloudBundesland" -}}
{{- required "Bundesland muss angegeben sein" (.Values.ozgcloud).bundesland }}
{{- end -}}

{{- define "app.ozgcloudBezeichner" -}}
{{- required "ozgcloud.bezeichner muss angegeben sein" (.Values.ozgcloud).bezeichner -}}
{{- if lt 46 (len (.Values.ozgcloud).bezeichner) -}}
{{ fail (printf "ozgcloud.bezeichner %s ist zu lang (max. 46 Zeichen)" (.Values.ozgcloud).bezeichner) }}
{{- end -}}
{{- end -}}

{{- define "app.ozgcloudEnvironment" -}}
{{- required "Environment muss angegeben sein" (.Values.ozgcloud).environment -}}
{{- end -}}

{{- define "app.ssoRealm" -}}
{{ printf "%s-%s-%s" (include "app.ozgcloudBundesland" .) ( include "app.ozgcloudBezeichner" . ) ( include "app.ozgcloudEnvironment" . ) | trunc 63 | trimSuffix "-" }}
{{- end -}}

{{- define "app.ssoClientName" -}}
{{- if (.Values.sso).client_name -}}
{{ printf "%s" (.Values.sso).client_name }}
{{- else -}}
{{ printf "alfa" }}
{{- end -}}
{{- end -}}

{{- define "app.ssoRealmDisplayName" -}}
{{- if (.Values.sso).displayName -}}
{{ printf "%s" (.Values.sso).displayName }}
{{- else if eq (include "app.ozgcloudEnvironment" . ) "prod" -}}
{{ printf "Realm für %s" (include "app.ozgcloudBezeichner" . | title ) }}
{{- else -}}
{{ printf "Realm für %s (%s)" (include "app.ozgcloudBezeichner" . | title ) (include "app.ozgcloudEnvironment" .) }}
{{- end -}}
{{- end -}}

{{- define "app.ssoServerUrl" -}}
{{- required "sso.serverUrl muss angegeben sein" (.Values.sso).serverUrl -}}
{{- end -}}

{{- define "app.baseDomain" -}}
{{- printf "%s.%s" (include "app.ozgcloudBezeichner" .) (include "app.baseUrl" .) }}
{{- end -}}

{{- define "app.generateKeycloakGroupRessourceName" -}}
{{- $kebabcaseGroupName := . | title | nospace | kebabcase }}
{{- $alphanumericGroupName :=  (include "app.replaceAllNonAlphanumeric" $kebabcaseGroupName ) }}
{{- printf "%s-keycloak-group"  ( $alphanumericGroupName | trunc 20) }}
{{- end -}}

{{- define "app.replaceAllNonAlphanumeric" -}}
{{- trimAll "-" ( regexReplaceAll "[^a-zA-Z0-9-]" . "" ) }}
{{- end -}}

{{- define "app.generateKeycloakUserRessourceName" -}}
{{- $lowerUserName := . | lower }}
{{- $alphanumericUserName :=  (include "app.replaceAllNonAlphanumeric" $lowerUserName ) }}
{{- printf "%s-keycloak-user"  ( $alphanumericUserName | trunc 20) }}
{{- end -}}

{{- define "app.generateKeycloakClientRessourceName" -}}
{{- $lowerClientName := . | lower }}
{{- $alphanumericClientName :=  (include "app.replaceAllNonAlphanumeric" $lowerClientName ) }}
{{- printf "%s-keycloak-client"  ( $alphanumericClientName | trunc 20) }}
{{- end -}}

{{- define "app.serviceAccountName" -}}
{{ printf "%s" ( (.Values.serviceAccount).name | default "alfa-service-account" ) }}
{{- end -}}